/* $Id: identifier.h,v 1.5 2009/01/27 15:40:22 potyra Exp $ 
 *
 * Copyright (C) 2007-2009 FAUcc Team <info@faumachine.org>.
 * This program is free software. You can redistribute it and/or modify it
 * under the terms of the GNU General Public License, either version 2 of
 * the License, or (at your option) any later version. See COPYING.
 */

#ifndef __IDENTIFIER_H_INCLUDED
#define __IDENTIFIER_H_INCLUDED

#define IDENTIFIER_HASH_SIZE	8192

extern unsigned int
identifier_hash(const char *name);

extern const char *
identifier_new(const char *name);

extern const char *
identifier_tmp(void);
extern int
identifier_is_tmp(const char *name);

extern const char *
identifier_dup(const char *name);

extern void
identifier_free(const char *name);

#endif /* __IDENTIFIER_H_INCLUDED */
